import pwd
import os
from charms.reactive import hook
from charmhelpers.fetch import apt_install
from charmhelpers.core.hookenv import status_set
from charmhelpers.core.templating import render
from charmhelpers.core import host
from charms.reactive import when, when_not
from charms.reactive import set_state, remove_state

@hook('install')
def install():
    apt_install(['ampache'])
    apt_install(['python3-mysql.connector'])
    set_state('ampache.available')


@when('ampache.available', 'database.available')
def setup_ampache(mysql):
    render(source='ampache.cfg.php',
           target='/etc/ampache/ampache.cfg.php',
           owner='www-data',
           perms=0o775,
           context={
               'db': mysql,
           })
    uid = pwd.getpwnam('www-data').pw_uid
    gid = pwd.getpwnam('www-data').pw_gid
    path = '/usr/share/ampache/www/'
    for root, dirs, files in os.walk(path):
        for d in dirs:
            os.chown(os.path.join(root, d), uid, gid)
        for f in files:
            os.chown(os.path.join(root, f), uid, gid)
    set_state('ampache.start')
    status_set('maintenance', 'Starting Ampache')


@when('database.connected', 'ampache.start')
@when_not('ampache.started')
def start_ampache(database):
    import mysql.connector
    from mysql.connector import Error
    with open('/etc/ampache/ampache.cfg.php', 'r', encoding='utf-8') as sfile:
        for line in sfile:
            if 'database_name' in line:
                db_name = line.split('=')[1].strip()[1:-1]
            if 'database_user' in line:
                db_user = line.split('=')[1].strip()[1:-1]
            if 'database_password' in line:
                db_password = line.split('=')[1].strip()[1:-1]
            if 'database_hostname' in line:
                db_hostname = line.split('=')[1].strip()[1:-1]
    
        try:
            cnx = mysql.connector.connect(host=db_hostname, database=db_name,
                                          user=db_user, password=db_password)
            if cnx.is_connected():
                with open('/usr/share/ampache/www/sql/ampache.sql', 'r', encoding='ISO-8859-1') as f:
                    sql = s = " ".join(filter(lambda x: not x.startswith('--') \
                                       and not x.startswith('/*') \
                                       and not x.isspace()
                                 , f.readlines() ))
                cursor = cnx.cursor()
                for result in cursor.execute(sql, params=None, multi=True):
                    pass
                cnx.commit()
                cursor.close()
        except Error as e:
            print(e)
        finally:
            cnx.close()

    #if not os.path.exists('/home/ubuntu/Music/'):
    #    os.makedirs('/home/ubuntu/Music/', 0775)

    assert host.service_restart('apache2'), 'Failed to start Apache2'
    set_state('ampache.started')


@when('ampache.available', 'ampache.started')
@when_not('ampache.start')
def stop_ampache():
    assert host.service_stop('apache2'), 'Failed to stop Apache2'
    remove_state('ampache.started')


@when('ampache.available')
@when_not('database.connected')
def missing_mysql():
    remove_state('ampache.start')
    status_set('blocked', 'Please add relation to MySQL')


@when('database.connected')
@when_not('database.available')
def waiting_mysql(mysql):
    remove_state('ampache.start')
    status_set('waiting', 'Waiting for MySQL')


@when('ampache.started')
def started():
    status_set('active', 'Ready')
